function [f, out] = make_fft(t, v, window, linear_out)
  %
  % v0.1 - MarkL @ eevblog,  Dec 23, 2014
  %
  % Compute the FFT of a signal given time (t) and voltage (v)
  % arrays.  Each are in row order (N rows x 1 column).
  %
  % Returns two row order arrays of of freqeuncy and dBV.
  %
  % If linear_out==1 then output units are in Vrms.  This is useful
  % for computing the linear average before converting to dB.
  % 
  % The default window is hanning, but can be overriden with any
  % other window function supported by the libraries.  If "none"
  % is given, no window is applied.
  %
  % Example usage:
  %
  %   [f, dbv] = make_fft(t, v);
  %   plot(f, dbv)
  %
  %   [f, vrms] = make_fft(t, v, "blackman", 1);
  %   plot(f, vrms)
  %
  SetDefaultValue(3, 'window', 'hanning');
  SetDefaultValue(4, 'linear_out', 0);

  n = numel(v);

  if (~strcmp(window, 'none'))
    eval(sprintf('w = %s(n);', window));

    % The reduction in signal level that occurs when applying the
    % window is reversed by multiplying by the inverse of the mean of
    % the window coefficients.
    %
    scale = 2/n/mean(w);

    if (linear_out)
      %out = abs(reshape(fft(scale*(w .* v)), n/2, 1))*sqrt(0.5);
      out = abs(fft(scale*(w .* v)))*sqrt(0.5);
      out = out(1:n/2);
    else
      %out = 20*log10(abs(reshape(fft(scale*(w .* v)), n/2, 1))*sqrt(0.5));
      out = 20*log10(abs(fft(scale*(w .* v)))*sqrt(0.5));
      out = out(1:n/2);
    end

  else
    % No window.
    %
    scale = 2/n;
    if (linear_out)
      out = abs(reshape(fft(scale*v), n/2, 1))*sqrt(0.5);
    else
      out = 20*log10(abs(reshape(fft(scale*v), n/2, 1))*sqrt(0.5));
    end
  end

  % Compute the frequency array.
  %
  f_inc = 1/(t(numel(t)) - t(1));
  f = ([0 : (n/2)-1] * f_inc)';

end
